@@ -28,25 +28,19 @@
 #include <gtk/gtk.h>
 #include <NvCtrlAttributes.h>
 
-#include "ctkimage.h"
+#include "glxinfo.h" /* xxx_abbrev functions */
+
+#include "ctkbanner.h"
 #include "ctkglx.h"
+#include "ctkutils.h"
 #include "ctkconfig.h"
 #include "ctkhelp.h"
 
 #include <GL/glx.h> /* GLX #defines */
 
 
-
-/* Prototypes */
-
-static void ctk_glx_probe_info(GtkWidget *widget,
-                               GtkObject *old_parent,
-                               gpointer data);
-
-
-
 /* Number of FBConfigs attributes reported in gui */
-#define NUM_FBCONFIG_ATTRIBS  30
+#define NUM_FBCONFIG_ATTRIBS  32
 
 
 /* FBConfig tooltips */
@@ -54,6 +48,11 @@ static const char * __fid_help  =
   "fid (Frame buffer ID) - Frame Buffer Configuration ID.";
 static const char * __vid_help  =
   "vid (XVisual ID) -  ID of the associated X Visual.";
+static const char * __vt_help  =
+  "vt (XVisual Type) -  Type of the associated X Visual. "
+  "Possible X visual types are 'tc', 'dc', 'pc', 'sc', 'gs', 'sg', '.' "
+  "which means TrueColor, DirectColor, PseudoColor, StaticColor, GrayScale, "
+  "StaticGray and None, respectively.";
 static const char * __bfs_help =
   "bfs (buffer size) - Number of bits per color in the color buffer.";
 static const char * __lvl_help =
@@ -102,8 +101,10 @@ static const char * __acb_help =
 static const char * __aca_help =
   "aca (Accumulator alpha size) - Number of bits per color used for alpha "
   "in the accumulator buffer.";
-static const char * __ms_help =
-  "ms (Multisample samples) - Number of samples per multisample.";
+static const char * __mvs_help =
+  "mvs (Multisample coverage samples) - Number of coverage samples per multisample.";
+static const char * __mcs_help =
+  "mcs (Multisample color samples) - Number of color samples per multisample.";
 static const char * __mb_help =
   "mb (Multisample buffer count) - Number of multisample buffers.";
 static const char * __cav_help =
@@ -157,32 +158,6 @@ GType ctk_glx_get_type(void)
 } /* ctk_glx_get_type() */
 
 
-static void add_table_row(GtkWidget *table,
-                          const gint row,
-                          const gint value_alignment,
-                          const gchar *name,
-                          const gchar *value)
-{
-    GtkWidget *label;
-
-
-    label = gtk_label_new(name);
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.0f);
-    gtk_label_set_selectable(GTK_LABEL(label), TRUE);
-
-    gtk_table_attach(GTK_TABLE(table), label, 0, 1, row, row + 1,
-                     GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-
-    label = gtk_label_new(value);
-    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-    gtk_misc_set_alignment(GTK_MISC(label), value_alignment, 0.0f);
-    gtk_label_set_selectable(GTK_LABEL(label), TRUE);
-
-    gtk_table_attach(GTK_TABLE(table), label, 1, 2, row, row + 1,
-                     GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-} /* add_table_row() */
-
-
 static void dummy_button_signal(GtkWidget *widget,
                                  gpointer user_data)
 {
@@ -199,6 +174,12 @@ static void dummy_button_signal(GtkWidget *widget,
  *       signal is thrown.  This will result in calling the
  *       ctk_glx_probe_info() function.
  */
+
+typedef struct WidgetSizeRec {
+    GtkWidget *widget;
+    int width;
+} WidgetSize;
+
 GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
                        CtkConfig *ctk_config, CtkEvent *ctk_event)
 {
@@ -207,38 +188,43 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
     GtkWidget *label;
     GtkWidget *banner;
     GtkWidget *hseparator;
-    GtkWidget *hbox, *hbox2;
+    GtkWidget *hbox, *hbox1;
     GtkWidget *vbox, *vbox2;
-    GtkWidget *table;
     GtkWidget *scrollWin;
     GtkWidget *event;    /* For setting the background color to white */
-    GdkColor   color;
-
+    GtkWidget *data_table, *header_table;
+    GtkWidget *data_viewport, *full_viewport;
+    GtkWidget *vscrollbar, *hscrollbar, *vpan;
+    GtkRequisition req;
     ReturnStatus ret;
 
     char * glx_info_str = NULL;               /* Test if GLX supported */
     GLXFBConfigAttr *fbconfig_attribs = NULL; /* FBConfig data */
     int i;                                    /* Iterator */
+    int num_fbconfigs = 0;
+    char *err_str = NULL;
 
     gchar *fbconfig_titles[NUM_FBCONFIG_ATTRIBS] = {
-        "fid",  "vid",  "bfs",  "lvl",
+        "fid",  "vid",  "vt", "bfs",  "lvl",
         "bf",   "db",   "st",
         "rs",   "gs",   "bs",   "as",
         "aux",  "dpt",  "stn",
         "acr",  "acg",  "acb",  "aca",
-        "ms",   "mb",
+        "mvs",  "mcs",  "mb",
         "cav",
         "pbw",  "pbh",  "pbp",
         "trt",  "trr",  "trg",  "trb",  "tra",  "tri"
     };
 
+    WidgetSize fbconfig_header_sizes[NUM_FBCONFIG_ATTRIBS];
+
     const char *fbconfig_tooltips[NUM_FBCONFIG_ATTRIBS] = {
-        __fid_help, __vid_help, __bfs_help, __lvl_help,
+        __fid_help, __vid_help, __vt_help, __bfs_help, __lvl_help,
         __bf_help,  __db_help,  __st_help,
         __rs_help,  __gs_help,  __bs_help,  __as_help,
         __aux_help, __dpt_help, __stn_help,
         __acr_help, __acg_help, __acb_help, __aca_help,
-        __ms_help,  __mb_help,
+        __mvs_help, __mcs_help, __mb_help,
         __cav_help,
         __pbw_help, __pbh_help, __pbp_help,
         __trt_help, __trr_help, __trg_help,
@@ -246,10 +232,6 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
     };
 
 
-    /* Parse a white color (for background) */
-    gdk_color_parse ("white", &color);
-
-
     /* Create the ctk glx object */
     object = g_object_new(CTK_TYPE_GLX, NULL);
     ctk_glx = CTK_GLX(object);
@@ -266,14 +248,15 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
 
     banner = ctk_banner_image_new(BANNER_ARTWORK_GLX);
     gtk_box_pack_start(GTK_BOX(ctk_glx), banner, FALSE, FALSE, 0);
-    
+
     /* Determine if GLX is supported */
     ret = NvCtrlGetStringAttribute(ctk_glx->handle,
                                    NV_CTRL_STRING_GLX_SERVER_VENDOR,
-                                   &glx_info_str );
+                                   &glx_info_str);
     free(glx_info_str);
     if ( ret != NvCtrlSuccess ) {
-        goto fail_glx_not_supported;
+        err_str = "Fail to query the GLX server vendor.";
+        goto fail;
     }
 
 
@@ -284,13 +267,14 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
     event = gtk_event_box_new();
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWin),
                                    GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
-    gtk_box_pack_start(GTK_BOX(ctk_glx), scrollWin, TRUE, TRUE, 0);
-    gtk_widget_modify_bg(event, GTK_STATE_NORMAL, &color);
+    gtk_widget_modify_fg(event, GTK_STATE_NORMAL, &(event->style->text[GTK_STATE_NORMAL]));
+    gtk_widget_modify_bg(event, GTK_STATE_NORMAL, &(event->style->base[GTK_STATE_NORMAL]));
     gtk_container_add(GTK_CONTAINER(event), hbox);
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollWin),
                                           event);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 5);
     ctk_glx->glxinfo_vpane = vbox;
+    gtk_widget_set_size_request(scrollWin, -1, 50);
 
 
     /* GLX 1.3 supports frame buffer configurations */
@@ -300,56 +284,108 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
     ret = NvCtrlGetVoidAttribute(handle, NV_CTRL_ATTR_GLX_FBCONFIG_ATTRIBS,
                                  (void *)(&fbconfig_attribs));
     if ( ret != NvCtrlSuccess ) {
-        goto fail_glx_not_supported;
+        err_str = "Failed to query list of GLX frame buffer configurations.";
+        goto fail;
     }
 
+    /* Count the number of fbconfigs */
+    if ( fbconfig_attribs ) {
+        for (num_fbconfigs = 0;
+             fbconfig_attribs[num_fbconfigs].fbconfig_id != 0;
+             num_fbconfigs++);
+    }
+    if ( ! num_fbconfigs ) {
+        err_str = "No frame buffer configurations found.";
+        
+        goto fail;
+    }
+
+
     /* Create clist in a scroll box */
-    hbox       = gtk_hbox_new(FALSE, 0);
+    hbox1      = gtk_hbox_new(FALSE, 0);
     label      = gtk_label_new("Frame Buffer Configurations");
     hseparator = gtk_hseparator_new();
-    gtk_box_pack_start(GTK_BOX(ctk_glx), hbox, FALSE, FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), hseparator, TRUE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox1), hseparator, TRUE, TRUE, 5);
 
     hbox      = gtk_hbox_new(FALSE, 0);
     vbox      = gtk_vbox_new(FALSE, 10);
     vbox2     = gtk_vbox_new(FALSE, 10);
-    scrollWin = gtk_scrolled_window_new(NULL, NULL);
-    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(scrollWin),
-                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-    gtk_box_pack_start(GTK_BOX(ctk_glx), GTK_WIDGET(vbox), TRUE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(vbox), scrollWin, TRUE, TRUE, 0);
+    vpan      = gtk_vpaned_new();
 
+    data_viewport = gtk_viewport_new(NULL, NULL);
+    gtk_widget_set_size_request(data_viewport, 400, 50);
+    vscrollbar = gtk_vscrollbar_new(gtk_viewport_get_vadjustment
+                                        (GTK_VIEWPORT(data_viewport)));
+    
+    full_viewport = gtk_viewport_new(NULL, NULL);
+    gtk_widget_set_size_request(full_viewport, 400, 50);
+    hscrollbar = gtk_hscrollbar_new(gtk_viewport_get_hadjustment
+                                     (GTK_VIEWPORT(full_viewport)));
     /*
      * NODE: Because clists have a hard time displaying tooltips in their
      *       column labels/buttons, we make the fbconfig table using a
      *       table widget.
      */
 
-    /* Count number of rows */
-    i = 1;
-    if ( fbconfig_attribs ) {
-        while ( fbconfig_attribs[i].fbconfig_id != 0 ) { i++; }
-    }
+    /* Create the header table */
 
-    table = gtk_table_new(i, NUM_FBCONFIG_ATTRIBS, FALSE);
-    event = gtk_event_box_new();
-    gtk_widget_modify_bg (table, GTK_STATE_NORMAL, &color);
-    gtk_container_add (GTK_CONTAINER(event), table);
-    gtk_widget_modify_bg(event, GTK_STATE_NORMAL, &color);
-    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollWin),
-                                          event);
+    header_table = gtk_table_new(num_fbconfigs, NUM_FBCONFIG_ATTRIBS, FALSE);
     
-    /* Generate table headings */
     for ( i = 0; i < NUM_FBCONFIG_ATTRIBS; i++ ) {
         GtkWidget * btn  = gtk_button_new_with_label(fbconfig_titles[i]);
         g_signal_connect(G_OBJECT(btn), "clicked",
                          G_CALLBACK(dummy_button_signal),
                          (gpointer) ctk_glx);
         ctk_config_set_tooltip(ctk_config, btn, fbconfig_tooltips[i]);
-        gtk_table_attach(GTK_TABLE(table), btn,  i, i+1,  0, 1,
+        gtk_table_attach(GTK_TABLE(header_table), btn,  i, i+1, 0, 1,
                          GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
+        
+        fbconfig_header_sizes[i].widget = btn;
+        gtk_widget_size_request(btn, &req);
+        fbconfig_header_sizes[i].width = req.width;
     }
+    
+    /* Create the data table */
+
+    data_table = gtk_table_new(num_fbconfigs, NUM_FBCONFIG_ATTRIBS, FALSE);
+    event = gtk_event_box_new();
+    
+    gtk_widget_modify_fg(data_table, GTK_STATE_NORMAL, 
+                         &(data_table->style->text[GTK_STATE_NORMAL]));
+    gtk_widget_modify_bg(data_table, GTK_STATE_NORMAL, 
+                         &(data_table->style->base[GTK_STATE_NORMAL]));
+    gtk_container_add (GTK_CONTAINER(event), data_table);
+    gtk_widget_modify_fg(event, GTK_STATE_NORMAL, 
+                         &(event->style->text[GTK_STATE_NORMAL]));
+    gtk_widget_modify_bg(event, GTK_STATE_NORMAL, 
+                         &(event->style->base[GTK_STATE_NORMAL]));
+    gtk_container_add(GTK_CONTAINER(data_viewport), event);
+
+    /* Pack the fbconfig header and data tables */
+
+    vbox = gtk_vbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), header_table, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), data_viewport, TRUE, TRUE, 0);
+    gtk_container_add (GTK_CONTAINER(full_viewport), vbox);
+
+    vbox = gtk_vbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), full_viewport, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hscrollbar, FALSE, FALSE, 0);
+    
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), vscrollbar, FALSE, FALSE, 0);
+    
+    vbox = gtk_vbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox1, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+    
+    gtk_paned_pack1 (GTK_PANED (vpan), scrollWin, TRUE, FALSE);
+    gtk_paned_pack2 (GTK_PANED (vpan), vbox, TRUE, FALSE);
+    gtk_box_pack_start(GTK_BOX(ctk_glx), vpan, TRUE, TRUE, 0);
+    
+    /* Fill the data table */
 
     if ( fbconfig_attribs ) {
 
@@ -372,6 +408,8 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
             } else {
                 sprintf((char *) (&(str[cell++])),".");
             }
+            snprintf((char *) (&(str[cell++])), 16, "%s",
+                     x_visual_type_abbrev(fbconfig_attribs[i].x_visual_type));
             snprintf((char *) (&(str[cell++])), 16, "%3d",
                      fbconfig_attribs[i].buffer_size);
             snprintf((char *) (&(str[cell++])), 16, "%2d",
@@ -404,8 +442,20 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
                      fbconfig_attribs[i].accum_blue_size);
             snprintf((char *) (&(str[cell++])), 16, "%2d",
                      fbconfig_attribs[i].accum_alpha_size);
-            snprintf((char *) (&(str[cell++])), 16, "%2d",
-                     fbconfig_attribs[i].multi_samples);
+            if (fbconfig_attribs[i].multi_sample_valid) {
+                snprintf((char *) (&(str[cell++])), 16, "%2d",
+                         fbconfig_attribs[i].multi_samples);
+                if (fbconfig_attribs[i].multi_sample_coverage_valid) {
+                    snprintf((char *) (&(str[cell++])), 16, "%2d",
+                             fbconfig_attribs[i].multi_samples_color);
+                } else {
+                    snprintf((char *) (&(str[cell++])), 16, "%2d",
+                             fbconfig_attribs[i].multi_samples);
+                }
+            } else {
+                snprintf((char *) (&(str[cell++])), 16, " 0");
+                snprintf((char *) (&(str[cell++])), 16, " 0");
+            }
             snprintf((char *) (&(str[cell++])), 16, "%1d",
                      fbconfig_attribs[i].multi_sample_buffers);
             snprintf((char *) (&(str[cell++])), 16, "%s",
@@ -433,10 +483,27 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
             /* Populate row cells */
             for ( cell = 0; cell < NUM_FBCONFIG_ATTRIBS ; cell++) {
                 GtkWidget * label = gtk_label_new( str[cell] );
-                gtk_table_attach(GTK_TABLE(table), label,
+                gtk_label_set_justify( GTK_LABEL(label), GTK_JUSTIFY_CENTER);
+                gtk_table_attach(GTK_TABLE(data_table), label,
                                  cell, cell+1, 
                                  i+1, i+2,
-                                 GTK_EXPAND, GTK_EXPAND, 3, 2);
+                                 GTK_EXPAND, GTK_EXPAND, 0, 0);
+
+                /* Make sure the table headers are the same width
+                 * as their table data column
+                 */
+                gtk_widget_size_request(label, &req);
+                
+                if ( fbconfig_header_sizes[cell].width > req.width ) {
+                    gtk_widget_set_size_request(label,
+                                                fbconfig_header_sizes[cell].width,
+                                                -1);
+                } else if ( fbconfig_header_sizes[cell].width < req.width ) {
+                    fbconfig_header_sizes[cell].width = req.width + 6;
+                    gtk_widget_set_size_request(fbconfig_header_sizes[cell].widget,
+                                                fbconfig_header_sizes[cell].width,
+                                                -1);
+                }
             }
             i++;
 
@@ -452,30 +519,21 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
 
     gtk_widget_show_all(GTK_WIDGET(object));
 
-    /* Set to probe for GLX info when widget is swapped in */
-    ctk_glx->probe_handler_id =
-        g_signal_connect(G_OBJECT(ctk_glx), "parent-set",
-                         G_CALLBACK(ctk_glx_probe_info),
-                         (gpointer) ctk_glx);
-
     return GTK_WIDGET(object);
 
 
     /* Failure (no GLX) */
- fail_glx_not_supported:
-    label = gtk_label_new("The OpenGL extension 'GLX' is not supported by\n"
-                          "the X server or there was a problem retrieving\n"
-                          "GLX information from the X server."
-                          );
-    gdk_color_parse ("#FF0000", &color);
-    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &color);
-    gtk_label_set_selectable(GTK_LABEL(label), TRUE);
+ fail:
+    if (err_str) {
+        label = gtk_label_new(err_str);
+        gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 
-    gtk_container_add(GTK_CONTAINER(ctk_glx), label);
+        gtk_container_add(GTK_CONTAINER(ctk_glx), label);
+    }
 
     /* Free memory that may have been allocated */
     free(fbconfig_attribs);
-
+    
     gtk_widget_show_all(GTK_WIDGET(object));
     return GTK_WIDGET(object);
 
@@ -487,11 +545,9 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
 /* Probes for GLX information and sets up the results
  * in the GLX widget.
  */
-static void ctk_glx_probe_info(GtkWidget *widget, 
-                               GtkObject *old_parent,
-                               gpointer data)
+void ctk_glx_probe_info(GtkWidget *widget)
 {
-    CtkGLX *ctk_glx = (CtkGLX *) data;
+    CtkGLX *ctk_glx = CTK_GLX(widget);
 
     ReturnStatus ret;
 
@@ -516,18 +572,15 @@ static void ctk_glx_probe_info(GtkWidget *widget,
     GtkWidget *table;
 
 
-    /* Make sure the widget was initialized */
-    if ( !ctk_glx || !ctk_glx->glxinfo_vpane || !ctk_glx->probe_handler_id ) {
+    /* Make sure the widget was initialized and that glx information
+     * has not yet been initialized.
+     */
+    if ( !ctk_glx || !ctk_glx->glxinfo_vpane ||
+         ctk_glx->glxinfo_initialized ) {
         return;
     }
 
 
-    /* Disconnect the handler */
-    g_signal_handler_disconnect(G_OBJECT(ctk_glx),
-                                ctk_glx->probe_handler_id);
-    ctk_glx->probe_handler_id = 0;
-
-
     /* Get GLX information */
     ret = NvCtrlGetStringAttribute(ctk_glx->handle,
                                    NV_CTRL_STRING_GLX_DIRECT_RENDERING,
@@ -619,9 +672,12 @@ static void ctk_glx_probe_info(GtkWidget *widget,
     table = gtk_table_new(2, 2, FALSE);
     gtk_table_set_row_spacings(GTK_TABLE(table), 3);
     gtk_table_set_col_spacings(GTK_TABLE(table), 15);
-    add_table_row(table, 0, 0, "Direct Rendering:", direct_rendering);
-    add_table_row(table, 1, 0, "GLX Extensions:",
-                  glx_extensions?glx_extensions:"");
+    add_table_row(table, 0,
+                  0, 0, "Direct Rendering:",
+                  0, 0,  direct_rendering);
+    add_table_row(table, 1,
+                  0, 0, "GLX Extensions:",
+                  0, 0,  glx_extensions);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), hbox2, FALSE, FALSE, 5);
     gtk_box_pack_start(GTK_BOX(hbox), table, FALSE, FALSE, 0);
@@ -640,12 +696,15 @@ static void ctk_glx_probe_info(GtkWidget *widget,
     table = gtk_table_new(3, 2, FALSE);
     gtk_table_set_row_spacings(GTK_TABLE(table), 3);
     gtk_table_set_col_spacings(GTK_TABLE(table), 15);
-    add_table_row(table, 0, 0, "Vendor:",
-                  server_vendor?server_vendor:"");
-    add_table_row(table, 1, 0, "Version:",
-                  server_version?server_version:"");
-    add_table_row(table, 2, 0, "Extensions:",
-                  server_extensions?server_extensions:"");
+    add_table_row(table, 0,
+                  0, 0, "Vendor:",
+                  0, 0, server_vendor);
+    add_table_row(table, 1,
+                  0, 0, "Version:",
+                  0, 0, server_version);
+    add_table_row(table, 2,
+                  0, 0, "Extensions:",
+                  0, 0, server_extensions);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), hbox2, FALSE, FALSE, 5);
     gtk_box_pack_start(GTK_BOX(hbox), table, FALSE, FALSE, 0);
@@ -664,12 +723,15 @@ static void ctk_glx_probe_info(GtkWidget *widget,
     table = gtk_table_new(3, 2, FALSE);
     gtk_table_set_row_spacings(GTK_TABLE(table), 3);
     gtk_table_set_col_spacings(GTK_TABLE(table), 15);
-    add_table_row(table, 0, 0, "Vendor:",
-                  client_vendor?client_vendor:"");
-    add_table_row(table, 1, 0, "Version:",
-                  client_version?client_version:"");
-    add_table_row(table, 2, 0, "Extensions:",
-                  client_extensions?client_extensions:"");
+    add_table_row(table, 0,
+                  0, 0, "Vendor:",
+                  0, 0, client_vendor);
+    add_table_row(table, 1,
+                  0, 0, "Version:",
+                  0, 0, client_version);
+    add_table_row(table, 2,
+                  0, 0, "Extensions:",
+                  0, 0, client_extensions);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), hbox2, FALSE, FALSE, 5);
     gtk_box_pack_start(GTK_BOX(hbox), table, FALSE, FALSE, 0);
@@ -689,10 +751,18 @@ static void ctk_glx_probe_info(GtkWidget *widget,
     vbox2 = gtk_vbox_new(FALSE, 0);
     gtk_table_set_row_spacings(GTK_TABLE(table), 3);
     gtk_table_set_col_spacings(GTK_TABLE(table), 15);
-    add_table_row(table, 0, 0, "Vendor:", opengl_vendor);
-    add_table_row(table, 1, 0, "Renderer:", opengl_renderer);
-    add_table_row(table, 2, 0, "Version:", opengl_version);
-    add_table_row(table, 3, 0, "Extensions:", opengl_extensions);
+    add_table_row(table, 0,
+                  0, 0, "Vendor:",
+                  0, 0, opengl_vendor);
+    add_table_row(table, 1,
+                  0, 0, "Renderer:",
+                  0, 0, opengl_renderer);
+    add_table_row(table, 2,
+                  0, 0, "Version:",
+                  0, 0, opengl_version);
+    add_table_row(table, 3,
+                  0, 0, "Extensions:",
+                  0, 0, opengl_extensions);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), hbox2, FALSE, FALSE, 5);
     gtk_box_pack_start(GTK_BOX(hbox), table, FALSE, FALSE, 0);
@@ -702,7 +772,7 @@ static void ctk_glx_probe_info(GtkWidget *widget,
     /* Show the information */
     gtk_widget_show_all(GTK_WIDGET(ctk_glx));
 
-
+    ctk_glx->glxinfo_initialized = True;
 
     /* Fall through */
  done:
@@ -803,6 +873,7 @@ GtkTextBuffer *ctk_glx_create_help(GtkTextTagTable *table,
                   "\t%s\n\n"
                   "\t%s\n\n"
                   "\t%s\n\n"
+                  "\t%s\n\n"
 
                   "\t%s\n\n"
                   "\t%s\n\n"
@@ -828,6 +899,7 @@ GtkTextBuffer *ctk_glx_create_help(GtkTextTagTable *table,
 
                   __fid_help,
                   __vid_help,
+                  __vt_help,
                   __bfs_help,
                   __lvl_help,
                   __bf_help,
@@ -845,7 +917,8 @@ GtkTextBuffer *ctk_glx_create_help(GtkTextTagTable *table,
                   __acg_help,
                   __acb_help,
                   __aca_help,
-                  __ms_help,
+                  __mvs_help,
+                  __mcs_help,
                   __mb_help,
 
                   __cav_help,
